pub mod types;

use std::io::Write;
use tool_helper::{Error, Input};
use types::{LPC, MonoADPCMIterator, VAGADPCM, VAGHeader};

pub fn convert(input: Input, output: &mut dyn Write) -> Result<(), Error> {
    let mut wav_file   = hound::WavReader::new(input)?;
    let     wav_header = wav_file.spec();

    validate(&wav_header)?;
    let mut samples      = Vec::new();
    let mut sample_count = 0;

    let mut lpc = LPC::empty();
    for adpcm_sample in MonoADPCMIterator::create(wav_file.samples::<i16>()) {
        let (vagadpcm, new_lpc) = VAGADPCM::create(adpcm_sample?, lpc);
        samples.push(vagadpcm);        
        lpc = new_lpc;

        sample_count += 1;
    }

    // TODO: Restructure
    tool_helper::raw::write_raw(output, &VAGHeader::create(sample_count, wav_header.sample_rate, "Planschi"))?;
    for sample in samples {
        tool_helper::raw::write_raw(output, &sample)?;
    }
    Ok(())
}

fn validate(wav_header: &hound::WavSpec) -> Result<(), Error> {
    if wav_header.sample_format != hound::SampleFormat::Int {
        return Err(Error::from_str("VAG: Only integer samples are supported as input."));
    }

    if wav_header.bits_per_sample != 16 {
        return Err(Error::from_str("VAG: Only 16bits samples are currently supported as input."));
    }

    if wav_header.channels != 1 {
        return Err(Error::from_str("VAG: Only mono samples are currently supported"));
    }

    Ok(())
}